unit SelectDirectoryDialog platform;

interface

uses
  Dialogs, Classes, Windows, ShlObj, ActiveX;

type
  TSelectDirectoryDialog = class(TCommonDialog)
  private
    fDirectory : String;
    fInitialDir : String;
    fInitialstring: String;
    fTitle : String;
  protected
  public
    function Execute:Boolean; override;
    constructor Create(AOwner:TComponent); override;
  published
    property Directory : String read fDirectory write fDirectory;
    property InitialDir : String read fInitialstring write fInitialString;
    property Title : String read fTitle write fTitle;
  end;

procedure Register;

implementation

{$R *.res}

procedure Register;
  begin
    RegisterComponents('Tools', [TSelectDirectoryDialog]);
  end;

function BrowseCallbackProc(Wnd: HWND; uMsg: UINT; lParam, lpData: LPARAM):integer; stdcall;
  begin
    case uMsg of
      BFFM_INITIALIZED : SendMessage(Wnd,BFFM_SETSELECTION,WPARAM(TRUE),lpData);
    end;
    Result:=0;
  end;

type
  PTaskWindow = ^TTaskWindow;
  TTaskWindow = record
    Next: PTaskWindow;
    Window: HWnd;
  end;

var
  TaskActiveWindow: HWnd = 0;
  TaskFirstWindow: HWnd = 0;
  TaskFirstTopMost: HWnd = 0;
  TaskWindowList: PTaskWindow = nil;

function DoDisableWindow(Window: HWnd; Data: Longint): Bool; stdcall;
  var
    P: PTaskWindow;
  begin
    if (Window <> TaskActiveWindow) and IsWindowVisible(Window) and
      IsWindowEnabled(Window) then
    begin
      New(P);
      P^.Next := TaskWindowList;
      P^.Window := Window;
      TaskWindowList := P;
      EnableWindow(Window, False);
    end;
    Result := True;
  end;

procedure EnableTaskWindows(WindowList: Pointer);
  var
    P: PTaskWindow;
  begin
    while WindowList <> nil do
    begin
      P := WindowList;
      if IsWindow(P^.Window) then EnableWindow(P^.Window, True);
      WindowList := P^.Next;
      Dispose(P);
    end;
  end;

function DisableTaskWindows(ActiveWindow: HWnd): Pointer;
  var
    SaveActiveWindow: HWND;
    SaveWindowList: Pointer;
  begin
    Result := nil;
    SaveActiveWindow := TaskActiveWindow;
    SaveWindowList := TaskWindowList;
    TaskActiveWindow := ActiveWindow;
    TaskWindowList := nil;
    try
      try
        EnumThreadWindows(GetCurrentThreadID, @DoDisableWindow, 0);
        Result := TaskWindowList;
      except
        EnableTaskWindows(TaskWindowList);
        raise;
      end;
    finally
      TaskWindowList := SaveWindowList;
      TaskActiveWindow := SaveActiveWindow;
    end;
  end;

function SelectDirectory(const Caption:string; const Root:WideString; SelectedDir:PChar; out Directory:WideString):Boolean;
  var
    WindowList: Pointer;
    BrowseInfo: TBrowseInfo;
    Buffer: PChar;
    RootItemIDList, ItemIDList: PItemIDList;
    ShellMalloc: IMalloc;
    IDesktopFolder: IShellFolder;
    Eaten, Flags: LongWord;
    ActiveWindow: HWND;
  begin
    Result:=False;
    Directory:='';
    FillChar(BrowseInfo,SizeOf(BrowseInfo), 0);
    if (ShGetMalloc(ShellMalloc)=S_OK) and (ShellMalloc<>nil) then begin
      Buffer:=ShellMalloc.Alloc(MAX_PATH);
      try
        RootItemIDList:=nil;
        if (Root<>'') then begin
          SHGetDesktopFolder(IDesktopFolder);
          IDesktopFolder.ParseDisplayName(0,nil,POleStr(Root),Eaten,RootItemIDList,Flags);
        end;
        with BrowseInfo do begin
          hwndOwner:=0;
          pidlRoot:=RootItemIDList;
          pszDisplayName:=Buffer;
          lpszTitle:=PChar(Caption);
          ulFlags:=BIF_RETURNONLYFSDIRS;
          if ((SelectedDir<>nil) and (SelectedDir<>'')) then begin
            lpfn:=BrowseCallbackProc;
            lParam:=integer(SelectedDir);
          end;
        end;
        ActiveWindow:=GetActiveWindow;
        WindowList:=DisableTaskWindows(0);
        try
          ItemIDList:=ShBrowseForFolder(BrowseInfo);
        finally
          EnableTaskWindows(WindowList);
          SetActiveWindow(ActiveWindow);
        end;
        Result:= (ItemIDList<>nil);
        if Result then begin
          ShGetPathFromIDList(ItemIDList,Buffer);
          ShellMalloc.Free(ItemIDList);
          Directory:=Buffer;
        end;
      finally
        ShellMalloc.Free(Buffer);
      end;
    end;
  end;

{ TSelectDirectoryDialog }

constructor TSelectDirectoryDialog.Create(AOwner:TComponent);
  begin
    inherited;
    fInitialDir:='';
    fDirectory:='';
    fTitle:='';
  end;

function TSelectDirectoryDialog.Execute:Boolean;
  var Dir : WideString;
  begin
    Dir:=fDirectory;
    Result:=SelectDirectory('Open',fInitialDir,PChar(fDirectory),Dir);
    fDirectory:=Dir;
  end;

end.

